#!/usr/bin/env python3
# coding=utf-8
"""
Air conditionner classes
"""

import logging

import bitstring

from eakon import HVAC
from eakon.enums import toshiba_enum


class Toshiba(HVAC):
    """
    Mitsubishi FG86
    """
    __HDR_FIRST_MARK = 4439
    __HDR_FIRST_SPACE = 4708
    __MARK = 562
    __ONE_SPACE = 1593
    __ZERO_SPACE = 521
    __temp_max = 30
    __temp_min = 16
    __start_mark = [__HDR_FIRST_MARK, __HDR_FIRST_SPACE]
    __repeat_mark = [__HDR_FIRST_SPACE, __HDR_FIRST_MARK, __HDR_FIRST_SPACE]

    frame = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, enum=toshiba_enum)
        self.one_mark = self.__MARK
        self.one_space = self.__ONE_SPACE
        self.zero_mark = self.__MARK
        self.zero_space = self.__ZERO_SPACE

    def _get_wave(self):
        wave = self.__start_mark.copy()
        for x in self._get_bitstring():
            wave.extend(self._get_zero() if x == "0" else self._get_one())
        wave.extend([self.__MARK])

        wave.extend(self.__repeat_mark)

        for x in self._get_bitstring():
            wave.extend(self._get_zero() if x == "0" else self._get_one())
        wave.extend([self.__MARK])

        wave.extend(self.__repeat_mark)

        for x in self._get_footer():
            wave.extend(self._get_zero() if x == "0" else self._get_one())
        wave.extend([self.__MARK])

        return wave

    def _get_bitstring(self):
        fmt = """
        uint: 8 = h1,
        uint: 8 = h2,
        uint: 4 = n5,
        uint: 4 = n6,
        uint: 4 = n7,
        uint: 4 = n8,
        uint: 4 = n9,
        uint: 4 = n10,
        uint: 4 = n11,
        uint: 4 = n12
        """

        data = {
            'h1': 0xc2,
            'h2': 0x3d,
            "n5": self._get_n5(),  # fan vertical mode ?
            "n6": 0xf,
            "n7": (~bitstring.Bits(uint=self._get_n5(), length=4)).uint,
            "n8": 0x0,
            "n9": self._get_temp_intcode(),
            "n10": self.mode.value,
            "n11": (~bitstring.Bits(uint=self._get_temp_intcode(), length=4)).uint,
            "n12": (~bitstring.Bits(uint=self.mode.value, length=4)).uint
        }

        return bitstring.pack(fmt, **data).bin

    def _get_n5(self):
        if self.mode == self.enum.Mode.AUTO or self.mode == self.enum.Mode.DRY:
            return 0x1
        else:
            return 0xb

    def _get_temp_intcode(self):
        temp_dict = {
            16: 0,  # WEIRD
            17: 0,
            18: 1,
            19: 3,
            20: 2,
            21: 6,
            22: 7,
            23: 5,
            24: 4,
            25: 12,
            26: 13,
            27: 9,
            28: 8,
            29: 10,
            30: 11
        }
        assert self.temperature in temp_dict.keys()
        return temp_dict[self.temperature]

    def _get_footer(self):
        fmt = """
        uint: 8 = b1,
        uint: 8 = b2,
        uint: 8 = b3,
        uint: 8 = b4,
        uint: 8 = b5,
        uint: 8 = b6,
        """

        def get_b6():
            if self.temperature > self.__temp_min:
                return 0x3a if self.mode in (self.enum.Mode.AUTO, self.enum.Mode.DRY) else 0x3b
            else:
                return 0x4a if self.mode in (self.enum.Mode.AUTO, self.enum.Mode.DRY) else 0x4b

        data = {
            'b1': 0xd5,
            'b2': 0x65 if self.mode in (self.enum.Mode.AUTO, self.enum.Mode.DRY) else 0x66,
            "b3": 0x00,
            "b4": 0x00 if self.temperature > self.__temp_min else 0x10,
            "b5": 0x00,
            "b6": get_b6(),
        }

        return bitstring.pack(fmt, **data).bin


def _test_toshiba(send_ir=False):
    from pap_logger import PaPLogger
    import sys
    import pigpio
    import AnaviInfraredPhat
    from time import sleep
    anavi_phat = None

    PaPLogger(level=logging.INFO, verbose_fmt=True)

    if send_ir:
        pi = pigpio.pi("node2", 8888)
        if not pi.connected:
            logging.error("Could not connect to pigpiod on node2")
            sys.exit(0)

        anavi_phat = AnaviInfraredPhat.IRSEND(pi, r"/proc/cpuinfo")

    for mode in [toshiba_enum.Mode.COOL, toshiba_enum.Mode.COOL, toshiba_enum.Mode.DRY, toshiba_enum.Mode.HEAT]:
        for temp in range(16, 31):
            hvac = Toshiba(temperature=temp, mode=mode)
            logging.info("{}_{} : {}".format(hvac.mode, hvac.temperature, hvac.bitstring))
            logging.info("{}_{} : {}".format(hvac.mode, hvac.temperature, hvac.wave))
            if send_ir:
                anavi_phat.send_ir(code=hvac.wave)
                sleep(5)


if __name__ == '__main__':
    _test_toshiba()



#!/usr/bin/env python3
# coding=utf-8
"""
Air conditionner classes
"""

import logging
import sys

import bitstring

from eakon import HVAC
from eakon.enums import daikin_enum


class Daikin(HVAC):
    """
    Daikin ARC478A5
    """
    __MARK = 433
    __ONE_SPACE = 1288
    __ZERO_SPACE = 440
    __temp_max = 30
    __temp_min = 16
    __init_mark = [__MARK, __ZERO_SPACE] * 5
    __start_mark = [__MARK, 25194, 3495, 1746]

    frame = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, enum=daikin_enum)
        self.one_mark = self.__MARK
        self.one_space = self.__ONE_SPACE
        self.zero_mark = self.__MARK
        self.zero_space = self.__ZERO_SPACE

    def _get_wave(self):
        wave = self.__init_mark.copy()
        wave.extend(self.__start_mark)

        for x in self._get_bitstring_frame1():
            wave.extend(self._get_zero() if x == "0" else self._get_one())
        wave.extend(self.__start_mark)

        for x in self._get_bitstring_frame2():
            wave.extend(self._get_zero() if x == "0" else self._get_one())
        wave.extend([self.__MARK])

        return wave

    def _get_bitstring(self):
        return self._get_bitstring_frame1() + self._get_bitstring_frame2()

    def _get_bitstring_frame1(self):
        fmt = """
        uint: 8 = h1,
        uint: 8 = h2,
        uint: 8 = h3,
        uint: 8 = b1,
        uint: 8 = b2,
        uint: 8 = b3,
        uint: 8 = b4,
        uint: 8 = b5,
        uint: 8 = b6,
        uint: 8 = b7,
        uint: 8 = b8,
        uint: 8 = b9,
        uint: 8 = b10,
        uint: 8 = b11,
        uint: 8 = b12,
        uint: 8 = b13,
        uint: 8 = b14,
        uint: 8 = b15,
        uint: 8 = b16,
        uint: 8 = cs
        """

        data = {
            'h1': 0x11,
            'h2': 0xda,
            'h3': 0x27,
            "b1": 0x00,
            "b2": 0x02,
            "b3": 0x00,
            "b4": 0x00,
            "b5": 0x00,
            "b6": 0x00,
            "b7": 0x00,
            "b8": 0x00,
            "b9": 0x00 if self.power == self.enum.Power.ON else 0x80,
            "b10": self.fan_vertical_mode.value if self.fan_vertical_mode != self.enum.FanVerticalMode.SWING else 0x0,
            "b11": 0x00,
            "b12": 0x00,
            "b13": 0x00,
            "b14": 0x00,
            "b15": 0x00,
            "b16": 0x00
        }

        self._reverse_endianness_and_get_checksum(data)

        return bitstring.pack(fmt, **data).bin

    def _get_bitstring_frame2(self):
        fmt = """
                uint: 8 = h1,
                uint: 8 = h2,
                uint: 8 = h3,
                uint: 8 = b18,
                uint: 8 = b19,
                uint: 8 = b20,
                uint: 8 = b21,
                uint: 8 = b22,
                uint: 8 = b23,
                uint: 8 = b24,
                uint: 8 = b25,
                uint: 8 = b26,
                uint: 8 = b27,
                uint: 8 = b28,
                uint: 8 = b29,
                uint: 8 = b30,
                uint: 8 = b31,        
                uint: 8 = b32,
                uint: 8 = cs,                
                """

        data = {
            'h1': 0x11,
            'h2': 0xda,
            'h3': 0x27,
            "b18": 0x00,
            "b19": 0x00,
            "b20": self._get_power_and_mode(),
            "b21": self._get_temp_intcode(),
            "b22": self._get_humidity_setting(),
            "b23": self._get_swing_and_force(),
            "b24": 0x00,
            "b25": 0x00,
            "b26": 0x06,
            "b27": 0x60,
            "b28": 0x00,
            "b29": 0x00,
            "b30": 0xc3,
            "b31": 0x00,
            "b32": 0x00
        }

        self._reverse_endianness_and_get_checksum(data)

        return bitstring.pack(fmt, **data).bin

    @staticmethod
    def _reverse_endianness_and_get_checksum(data):
        checksum = 0
        for b in data.values():
            checksum += b
        for k, b in data.items():
            b = bitstring.Bits(uint=b, length=8)
            b._reverse()
            data[k] = b.uint

        try:
            cs = bitstring.Bits(uint=checksum & 0xff, length=8)
        except bitstring.CreationError:
            from pprint import pformat
            logging.exception("Checksum value before reverse was {}".format(checksum))
            logging.exception("Checksum value before reverse after masking : {}".format(checksum & 0xff))
            logging.exception("data was\r\n{}".format(pformat(data)))
            sys.exit(0)
        cs._reverse()
        checksum = {
            "cs": cs.uint
        }
        data.update(checksum)

    def _get_temp_intcode(self):
        if self.mode in (self.enum.Mode.COOL, self.enum.Mode.HEAT):
            return self.temperature * 2
        elif self.mode in (self.enum.Mode.DRY, self.enum.Mode.AUTO):
            return 0x3  # TODO : review
        else:
            return 0xc  # TODO : review

    def _get_power_and_mode(self):
        return (bitstring.Bits(uint=self.mode.value, length=4) + bitstring.Bits(uint=self.power.value, length=4)).uint

    def _get_humidity_setting(self):
        if self.mode in (self.enum.Mode.COOL, self.enum.Mode.FAN):
            return 0x0
        else:
            # more settings available according to http://nandra.segv.jp/ir-remote/daikin2.html
            # not available on ARC478A5, so skipping code, setting to auto
            return 0x8

    def _get_swing_and_force(self):
        swing = 0xf if self.fan_vertical_mode == self.enum.FanVerticalMode.SWING else 0x0
        return (bitstring.Bits(uint=self.fan_power.value, length=4) + bitstring.Bits(uint=swing, length=4)).uint


def _test_daikin(send_ir=False):
    from pap_logger import PaPLogger
    import sys
    import pigpio
    import AnaviInfraredPhat
    from time import sleep

    PaPLogger(level=logging.INFO, verbose_fmt=True)
    hvac = Daikin()
    hvac.power = daikin_enum.Power.ON
    hvac.temperature = 21
    hvac.mode = daikin_enum.Mode.COOL
    hvac.fan_vertical_mode = daikin_enum.FanVerticalMode.TOP

    logging.info("\r{}".format(hvac))
    logging.info(hvac.bitstring)
    logging.info(hvac.wave)

    hvac.power = daikin_enum.Power.OFF
    logging.info("\r{}".format(hvac))
    logging.info(hvac.bitstring)
    logging.info(hvac.wave)
    if send_ir:
        pi = pigpio.pi("node1", 8888)
        if not pi.connected:
            logging.error("Could not connect to pigpiod on node2")
            sys.exit(0)
        anavi_phat = AnaviInfraredPhat.IRSEND(pi, r"/proc/cpuinfo")
        hvac = Daikin()
        hvac.power = daikin_enum.Power.ON

        temp = 21
        for mode in [daikin_enum.Mode.AUTO, daikin_enum.Mode.COOL, daikin_enum.Mode.DRY, daikin_enum.Mode.HEAT]:
            for vert in (
                    daikin_enum.FanVerticalMode.TOP, daikin_enum.FanVerticalMode.TOP_MIDDLE,
                    daikin_enum.FanVerticalMode.MIDDLE,
                    daikin_enum.FanVerticalMode.BOTTOM_MIDDLE, daikin_enum.FanVerticalMode.BOTTOM,
                    daikin_enum.FanVerticalMode.SWING):
                for fp in (daikin_enum.FanPower.AUTO, daikin_enum.FanPower.FORCE1, daikin_enum.FanPower.FORCE2,
                           daikin_enum.FanPower.FORCE3, daikin_enum.FanPower.FORCE4, daikin_enum.FanPower.FORCE5,
                           daikin_enum.FanPower.QUIET):
                    hvac.temperature = temp
                    hvac.mode = mode
                    hvac.fan_vertical_mode = vert
                    hvac.fan_power = fp
                    logging.info("\r{}".format(hvac))
                    logging.info(hvac.bitstring)
                    logging.info(hvac.wave)
                    anavi_phat.send_ir(code=hvac.wave)
                    sleep(15)
                    temp += 1

        hvac.mode = daikin_enum.Mode.AUTO
        hvac.fan_vertical_mode = daikin_enum.FanVerticalMode.TOP
        hvac.fan_power = daikin_enum.FanPower.AUTO
        anavi_phat.send_ir(code=hvac.wave)

        sleep(15)
        hvac.power = daikin_enum.Power.OFF
        logging.info("\r{}".format(hvac))
        logging.info(hvac.bitstring)
        logging.info(hvac.wave)
        anavi_phat.send_ir(code=hvac.wave)


if __name__ == '__main__':
    _test_daikin()



#!/usr/bin/env python3
# coding=utf-8
"""
Air conditionner classes
"""
import abc
import json
import logging
from abc import ABC
from importlib import import_module
from pathlib import Path

from eakon.enums import common_enum


class HVAC:
    """
    Parent class for air conditionner
    """
    __temp_max = 40
    __temp_min = 10
    __enum = common_enum

    one_mark = None
    one_space = None
    zero_mark = None
    zero_space = None
    pi = None

    def __init__(self, power=None, mode=None, temperature=None, wide_vanne_mode=None, area_mode=None, fan_power=None,
                 fan_high_power=None, fan_long=None, fan_vertical_mode=None, fan_horizontal_mode=None,
                 save_on_update=False, restore=False, enum=common_enum):

        self.__name = type(self).__name__
        self.enum = enum
        self._enums_dict = None

        if restore:
            self.restore()

        self._mode = self.enum.Mode.UNDEFINED
        self._wide_vanne_mode = None
        self._area_mode = None
        self._fan_power = None
        self._fan_high_power = None
        self._fan_long = None
        self._fan_vertical_mode = None
        self._fan_horizontal_mode = None
        self._power = None
        self._temperature = None
        self._save_on_update = False

        self.power = power
        self.mode = mode
        self.temperature = temperature
        if wide_vanne_mode:
            self.wide_vanne_mode = wide_vanne_mode
        if area_mode:
            self.area_mode = area_mode
        if fan_power:
            self.fan_power = fan_power
        if fan_high_power:
            self.fan_high_power = fan_high_power
        if fan_long:
            self.fan_long = fan_long
        if fan_vertical_mode:
            self.fan_vertical_mode = fan_vertical_mode
        if fan_horizontal_mode:
            self.fan_horizontal_mode = fan_horizontal_mode
        self.save_on_update = save_on_update

    def to_dict(self):
        """
        Stores the current state in a dictionnary
        :return: dict
        """
        return {
            "mode": "Mode.{}".format(self.mode.name),
            "wide_vanne_mode": "WideVanneMode.{}".format(self.wide_vanne_mode.name),
            "area_mode": "AreaMode.{}".format(self.area_mode.name),
            "fan_power": "FanPower.{}".format(self.fan_power.name),
            "fan_high_power": "FanHighPower.{}".format(self.fan_high_power.name),
            "fan_long": "FanLong.{}".format(self.fan_long.name),
            "fan_vertical_mode": "FanVerticalMode.{}".format(self.fan_vertical_mode.name),
            "fan_horizontal_mode": "FanHorizontalMode.{}".format(self.fan_horizontal_mode.name),
            "power": "Power.{}".format(self.power.name),
            "temperature": "Temperature.{}".format(self.temperature)
        }

    def restore(self):
        """
        restore the state of the class from file.
        """
        json_file = Path("/tmp/") / "eakon_{}.json".format(self.__name)
        try:
            if json_file.exists():
                logging.info("loading state from {}".format(json_file))
                hvac_dict = json.loads(json_file.read_text())
                for k, v in hvac_dict.items():
                    if isinstance(v, int):
                        val = v
                    else:
                        split = v.split(".")
                        val = getattr(self.enum, split[0])[split[1]]
                    self.__setattr__(k, val)
            else:
                logging.warning("failed to load from {} : file doesn't exists.".format(json_file))
        except IOError:
            logging.exception("failed to load from {}".format(json_file))
        except Exception as e:
            logging.exception(e)

    def save(self):
        """
        Saves the current state in a json file
        """
        if self._save_on_update:
            json_file = Path("/tmp/") / "{}.json".format(self.__name)
            try:
                state = self.to_dict()
                state.pop("power")
                json_file.write_text(json.dumps(state))
                logging.info(f"save state to {json_file}")
            except IOError:
                logging.exception(f"failed to save {json_file}")

    def __str__(self):
        rtn = "Model :\t\t\t\t\t{}\n".format(self.__name)
        rtn += "power :\t\t\t\t\t{}\n".format(self.enum.Power(self.power).name)
        rtn += "mode :\t\t\t\t\t{}\n".format(self.enum.Mode(self.mode).name)
        rtn += "temperature :\t\t\t{}\u00B0C\n".format(self.temperature)
        rtn += "wide_vanne_mode :\t\t{}\n".format(self.enum.WideVanneMode(self.wide_vanne_mode).name)
        rtn += "area_mode :\t\t\t\t{}\n".format(self.enum.AreaMode(self.area_mode).name)
        rtn += "fan_power :\t\t\t\t{}\n".format(self.enum.FanPower(self.fan_power).name)
        rtn += "fan_high_power :\t\t{}\n".format(self.enum.FanHighPower(self.fan_high_power).name)
        rtn += "fan_vertical_mode :\t\t{}\n".format(self.enum.FanVerticalMode(self.fan_vertical_mode).name)
        rtn += "fan_horizontal_mode :\t{}\n".format(self.enum.FanHorizontalMode(self.fan_horizontal_mode).name)
        rtn += "fan_long :\t\t\t\t{}\n".format(self.enum.FanLong(self.fan_long).name)
        return rtn

    @property
    def power(self):
        """
        Get/Set the power
        :return: Power
        """
        return self._power if self._power else None

    @power.setter
    def power(self, power):
        if power:
            if not isinstance(power, self.enum.Power):
                raise TypeError('must be an instance of Power Enum')
            self._power = power
            self.save()

    @property
    def mode(self):
        """
        Get/Set the mode
        :return: Mode
        """
        return self._mode if self._mode else self.enum.Mode.UNDEFINED

    @mode.setter
    def mode(self, mode):
        if mode:
            if not isinstance(mode, self.enum.Mode):
                raise TypeError('must be an instance of Mode Enum')
            self._mode = mode
            self.save()

    @property
    def temperature(self) -> int:
        """
        Get/Set the temperature
        :return: int
        """
        return self._temperature if self._temperature else None

    @temperature.setter
    def temperature(self, temperature: int):
        if temperature:
            if temperature < self.__temp_min:
                self._temperature = self.__temp_min
            elif temperature > self.__temp_max:
                self._temperature = self.__temp_max
            else:
                self._temperature = temperature
            self.save()

    @property
    def wide_vanne_mode(self):
        """
        Get/Set the wide vanne mode
        :return: WideVanneMode
        """
        return self._wide_vanne_mode if self._wide_vanne_mode else self.enum.WideVanneMode.UNDEFINED

    @wide_vanne_mode.setter
    def wide_vanne_mode(self, wide_vanne_mode):
        if wide_vanne_mode:
            if not isinstance(wide_vanne_mode, self.enum.WideVanneMode):
                raise TypeError('must be an instance of WideVanneMode Enum')
            self._wide_vanne_mode = wide_vanne_mode
            self.save()

    @property
    def area_mode(self):
        """
        Get/Set the area mode
        :return: AreaMode
        """
        return self._area_mode if self._area_mode else self.enum.AreaMode.UNDEFINED

    @area_mode.setter
    def area_mode(self, area_mode):
        if area_mode:
            if not isinstance(area_mode, self.enum.AreaMode):
                raise TypeError('must be an instance of AreaMode Enum')
            self._area_mode = area_mode
            self.save()

    @property
    def fan_power(self):
        """
        Get/Set the fan mode
        :return: FanPower
        """
        return self._fan_power if self._fan_power else self.enum.FanPower.UNDEFINED

    @fan_power.setter
    def fan_power(self, fan_power):
        if fan_power:
            if not isinstance(fan_power, self.enum.FanPower):
                raise TypeError('must be an instance of FanPower Enum')
            self._fan_power = fan_power
            self.save()

    @property
    def fan_high_power(self):
        """
        Get/Set the high power
        :return: FanHighPower
        """
        return self._fan_high_power if self._fan_high_power else self.enum.FanHighPower.UNDEFINED

    @fan_high_power.setter
    def fan_high_power(self, fan_high_power):
        if fan_high_power:
            if not isinstance(fan_high_power, self.enum.FanHighPower):
                raise TypeError('must be an instance of FanHighPower Enum')
            self._fan_high_power = self.enum.FanHighPower.NOT_AVAILABLE
            self.save()

    @property
    def fan_long(self):
        """
        Get/Set the Long Fan setting
        :return: FanLong
        """
        return self._fan_long if self._fan_long else self.enum.FanLong.UNDEFINED

    @fan_long.setter
    def fan_long(self, fan_long):
        if fan_long:
            if not isinstance(fan_long, self.enum.FanLong):
                raise TypeError('must be an instance of FanLong Enum')
            self._fan_long = self.enum.FanLong.NOT_AVAILABLE
            self.save()

    @property
    def fan_vertical_mode(self):
        """
        Get/Set the high power
        :return: FanVerticalMode
        """
        return self._fan_vertical_mode if self._fan_vertical_mode else self.enum.FanVerticalMode.UNDEFINED

    @fan_vertical_mode.setter
    def fan_vertical_mode(self, fan_vertical_mode):
        if fan_vertical_mode:
            if not isinstance(fan_vertical_mode, self.enum.FanVerticalMode):
                raise TypeError('must be an instance of FanVerticalMode Enum')
            self._fan_vertical_mode = fan_vertical_mode
            self.save()
        else:
            self._fan_vertical_mode = self.enum.FanVerticalMode.UNDEFINED

    @property
    def fan_horizontal_mode(self):
        """
        Get/Set the high power
        :return: FanHorizontalMode
        """
        return self._fan_horizontal_mode if self._fan_horizontal_mode else self.enum.FanHorizontalMode.UNDEFINED

    @fan_horizontal_mode.setter
    def fan_horizontal_mode(self, fan_horizontal_mode):
        if fan_horizontal_mode:
            if not isinstance(fan_horizontal_mode, self.enum.FanHorizontalMode):
                raise TypeError('must be an instance of FanHorizontalMode Enum')
            self._fan_horizontal_mode = fan_horizontal_mode
            self.save()
        else:
            self._fan_horizontal_mode = self.enum.FanHorizontalMode.UNDEFINED

    @property
    def save_on_update(self):
        """
        Get/Set the flag to save any changes to disk
        :return:
        """
        return self.__save_on_update

    @save_on_update.setter
    def save_on_update(self, save_on_update):
        if save_on_update is not None:
            if not isinstance(save_on_update, bool):
                raise TypeError('must be an instance of bool')
            self.__save_on_update = save_on_update

    def _get_one(self):
        return [self.one_mark, self.one_space]

    def _get_zero(self):
        return [self.zero_mark, self.zero_space]

    @abc.abstractmethod
    def _get_bitstring(self):
        pass

    @abc.abstractmethod
    def _get_wave(self):
        pass

    @property
    def bitstring(self):
        """
        returns the bits before encoding to a wave chain
        :return:
        """
        return self._get_bitstring()

    @property
    def wave(self):
        """
        returns a wave chain
        :return:
        """
        return self._get_wave()

    @property
    def enums(self):
        if not self._enums_dict:
            self._enums_dict = self.enum.get_enums_dict()
        return self._enums_dict


def get_eakon_instance_by_model(model_name) -> HVAC:
    """
    A helper function to instantiate a new class using a model name.
    :param model_name:
    :return:
    """
    model_name = model_name.lower()
    class_name = model_name.capitalize()
    try:
        import_module(model_name)
        model_class = getattr(import_module(model_name), class_name)
        return ABC.register(type(class_name, (model_class,), {}))()
    except (ModuleNotFoundError, TypeError) as exc:
        logging.debug("Exception was {}".format(exc))
        raise NotImplementedError(
            "No module {} implementing class {} was found. Model {} is unsupported.".format(model_name, class_name,
                                                                                            model_name))


if __name__ == '__main__':
    from pap_logger import PaPLogger

    PaPLogger(level=logging.INFO, verbose_fmt=True)
    for model in ["toshiba", "hitachi", "daikin", "toto"]:
        try:
            e = get_eakon_instance_by_model(model)
            e.power = e.enums["Power"].ON
            e.temperature = 26
            e.mode = e.enums["Mode"].COOL
            logging.info("\r\n{}".format(e))
            logging.info(e.bitstring)
        except NotImplementedError as e:
            logging.warning(e)



#!/usr/bin/env python3
# coding=utf-8
"""
Hitachi air conditionner classes
"""
import logging

import bitstring

from eakon import HVAC
from eakon.enums import hitachi_enum


class Hitachi(HVAC):
    """
    Mitsubishi FG86
    """
    __HDR_FIRST_MARK = 29785
    __HDR_FIRST_SPACE = 49362
    __HDR_SECOND_MARK = 3388
    __HDR_SECOND_SPACE = 1657
    __MARK = 428
    __ONE_SPACE = 1245
    __ZERO_SPACE = 410
    __temp_max = 32
    __temp_min = 16
    __mark = [__HDR_FIRST_MARK, __HDR_FIRST_SPACE, __HDR_SECOND_MARK, __HDR_SECOND_SPACE]

    frame = None

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs, enum=hitachi_enum)
        self.one_mark = self.__MARK
        self.one_space = self.__ONE_SPACE
        self.zero_mark = self.__MARK
        self.zero_space = self.__ZERO_SPACE

    def _get_wave(self):

        wave = self.__mark.copy()
        for x in self._get_bitstring():
            wave.extend(self._get_zero() if x == "0" else self._get_one())
        wave.extend([self.__MARK])
        return wave

    def _get_bitstring(self):
        fmt = """
        uint: 8 = h1,
        uint: 8 = h2,
        uint: 8 = h3,
        uint: 8 = b1,
        uint: 8 = b2,
        uint: 8 = b3,
        uint: 8 = b4,
        uint: 8 = b5,
        uint: 8 = b6,
        uint: 8 = b7,
        uint: 8 = b8,
        uint: 8 = b9,
        uint: 8 = b10,
        uint: 8 = b11,
        uint: 8 = b12,
        uint: 8 = b13,
        uint: 8 = b14,
        uint: 8 = b15,
        uint: 8 = b16,
        uint: 8 = b17,
        uint: 8 = b18,
        uint: 8 = b19,
        uint: 8 = b20,
        uint: 8 = b21,
        uint: 8 = b22,
        uint: 8 = b23,
        uint: 8 = b24,
        uint: 8 = b25,
        uint: 8 = b26,
        uint: 8 = b27,
        uint: 8 = b28,
        uint: 8 = b29,
        uint: 8 = b30,
        uint: 8 = b31,
        uint: 8 = b32,
        uint: 8 = b33,
        uint: 8 = b34,
        uint: 8 = b35,
        uint: 8 = b36,
        uint: 8 = b37,
        uint: 8 = b38,
        uint: 8 = b39,
        uint: 8 = b40,
        uint: 8 = b41,
        uint: 8 = b42,
        uint: 8 = b43,
        uint: 8 = b44,
        uint: 8 = b45,
        uint: 8 = b46,
        uint: 8 = b47,
        uint: 8 = b48,
        uint: 8 = b49,
        uint: 8 = b50
        """

        header = {'h1': 0x80,
                  'h2': 0x08,
                  'h3': 0x00
                  }
        pre_data = {
            1: 0x02,
            3: 0xff,
            5: 0x33,
            7: 0x49,
            9: 0xc2 if self.temperature == self.__temp_min else 0x22,
            11: self._get_temp_intcode(),
            13: 0x00,
            15: 0x00,
            17: 0x00,
            19: 0x00,
            21: 0x00,
            23: self.mode.value,
            25: 0x0f,  # AH AH
            27: 0x00,
            29: 0x00,
            31: 0x01,
            33: 0xc0,
            35: 0x80,
            37: 0x11,
            39: 0x00,
            41: 0x00,
            43: 0xff,
            45: 0xff,
            47: 0xff,
            49: 0xff,
        }
        data = {}
        for k, v in pre_data.items():
            data["b{}".format(k)] = v
            data["b{}".format(k + 1)] = (~bitstring.Bits(uint=v, length=8)).uint

        bitstring_dict = header
        bitstring_dict.update(data)
        return bitstring.pack(fmt, **bitstring_dict).bin

    def _get_temp_intcode(self):
        temp = bitstring.pack('uint:4=temp',
                              **{'temp': 0 if self.temperature == self.__temp_max else self.temperature - 16})
        temp.reverse()
        return bitstring.pack('uint:2=zero,uint:4=inv_temp,uint:2=max_temp',
                              **{'zero': 0,
                                 'inv_temp': temp.uint,
                                 'max_temp': 1 if self.temperature == self.__temp_max else 2}).int


def _test_hitachi(send_ir=False):
    from pap_logger import PaPLogger
    import sys
    import pigpio
    import AnaviInfraredPhat
    from time import sleep
    anavi_phat = None

    PaPLogger(level=logging.INFO, verbose_fmt=True)

    if send_ir:

        pi = pigpio.pi("node0", 8888)
        if not pi.connected:
            logging.error("Could not connect to pigpiod on node0")
            sys.exit(0)

        anavi_phat = AnaviInfraredPhat.IRSEND(pi, r"/proc/cpuinfo")

    for mode in [hitachi_enum.Mode.COOL, hitachi_enum.Mode.DRY, hitachi_enum.Mode.HEAT]:
        for temp in range(16, 33):
            hvac = Hitachi(temperature=temp, mode=mode)
            logging.info("{}_{} : {}".format(hvac.mode, hvac.temperature, hvac.bitstring))
            logging.info("{}_{} : {}".format(hvac.mode, hvac.temperature, hvac.wave))
            if send_ir:
                anavi_phat.send_ir(code=hvac.wave)
                sleep(5)


if __name__ == '__main__':
    _test_hitachi()
